library compassion;

import 'package:flutter/material.dart';

typedef ArgumentedRoute = Widget Function(
  BuildContext context,
  Object argument,
);

/// Compass provides a simpler interface to manage named routes with arguments.
class Compass {
  /// Build a [Compass] with `routes` initially, more can be added at runtime.
  /// Pass this [Compass] to `onGenerateRoute` for it to take effect.
  const Compass(this.routes);

  /// Contains the routes that will be matched by the navigator.
  final Map<String, ArgumentedRoute> routes;

  /// Method called by `onGenerateRoute`, processes a [RouteSettings]
  /// and calls the corresponding route with the provided arguments.
  Route<dynamic> call(RouteSettings settings) => MaterialPageRoute(
        settings: settings,
        builder: (context) => CompassProvider(
              compass: this,
              child: routes[settings.name](
                context,
                settings.arguments,
              ),
            ),
      );

  /// Finds and returns the closest compass in the tree.
  static Compass of(BuildContext context, {bool nullOk = false}) {
    final maybeCompass = context.inheritFromWidgetOfExactType(CompassProvider);
    if (maybeCompass is CompassProvider) {
      return maybeCompass._compass;
    }
    if (!nullOk) {
      throw Exception('Unable to find a Compass in the tree.');
    }
    return null;
  }
}

/// Provides a [Compass] to a tree.
/// Should only be used if you only use un-named routes,
/// or named routes outside of the context of [Compass].
class CompassProvider extends InheritedWidget {
  /// Creates a [CompassProvider], to be placed as a parent of the [Navigator]
  /// to which a [Compass] is passed
  const CompassProvider({Compass compass, Widget child})
      : _compass = compass,
        super(child: child);

  final Compass _compass;

  @override
  bool updateShouldNotify(InheritedWidget oldWidget) => false;
}
